@@ -39,6 +39,7 @@
 using namespace std;
 
 
+
 /**
  * Extract all capture groups of the first/last occurrence
  * of a regex pattern in each string
@@ -67,13 +68,16 @@ using namespace std;
  *
  * @version 1.4.7 (Marek Gagolewski, 2020-08-24)
  *    Use StriContainerRegexPattern::getRegexOptions
+ *
+ * @version 1.7.1 (Marek Gagolewski, 2021-06-19)
+ *    #153: named capture groups
  */
 SEXP stri__match_firstlast_regex(SEXP str, SEXP pattern, SEXP cg_missing, SEXP opts_regex, bool first)
 {
     // @TODO: capture_groups arg (integer vector/set - which capture groups to extract)
-    PROTECT(str = stri_prepare_arg_string(str, "str")); // prepare string argument
-    PROTECT(pattern = stri_prepare_arg_string(pattern, "pattern")); // prepare string argument
-    PROTECT(cg_missing = stri_prepare_arg_string_1(cg_missing, "cg_missing"));
+    PROTECT(str = stri__prepare_arg_string(str, "str")); // prepare string argument
+    PROTECT(pattern = stri__prepare_arg_string(pattern, "pattern")); // prepare string argument
+    PROTECT(cg_missing = stri__prepare_arg_string_1(cg_missing, "cg_missing"));
     R_len_t vectorize_length = stri__recycling_rule(true, 2, LENGTH(str), LENGTH(pattern));
 
     StriRegexMatcherOptions pattern_opts =
@@ -89,9 +93,9 @@ SEXP stri__match_firstlast_regex(SEXP str, SEXP pattern, SEXP cg_missing, SEXP o
     vector< vector< pair<const char*, const char*> > > occurrences(vectorize_length);
     R_len_t occurrences_max = 1;
 
+    StriContainerRegexPattern pattern_cont(pattern, (LENGTH(str)>0)?vectorize_length:LENGTH(pattern), pattern_opts);
     if (LENGTH(str) == 0 && LENGTH(pattern) > 0) {
         // we need to determine the number of capture groups anyway
-        StriContainerRegexPattern pattern_cont(pattern, LENGTH(pattern), pattern_opts);
         for (R_len_t i = pattern_cont.vectorize_init();
                 i != pattern_cont.vectorize_end();
                 i = pattern_cont.vectorize_next(i))
@@ -109,7 +113,6 @@ SEXP stri__match_firstlast_regex(SEXP str, SEXP pattern, SEXP cg_missing, SEXP o
     }
     else
     {
-        StriContainerRegexPattern pattern_cont(pattern, vectorize_length, pattern_opts);
         for (R_len_t i = pattern_cont.vectorize_init();
                 i != pattern_cont.vectorize_end();
                 i = pattern_cont.vectorize_next(i))
@@ -178,10 +181,18 @@ SEXP stri__match_firstlast_regex(SEXP str, SEXP pattern, SEXP cg_missing, SEXP o
                 SET_STRING_ELT(ret, i+j*vectorize_length, cg_missing);
         }
     }
+
+    if (pattern_cont.get_n() == 1) {  // only if there's 1 pattern, otherwise how to agree names?
+        SEXP dimnames;
+        STRI__PROTECT(dimnames = pattern_cont.getCaptureGroupRDimnames(0));  // reuses last matcher btw
+        if (!isNull(dimnames)) Rf_setAttrib(ret, R_DimNamesSymbol, dimnames);
+        STRI__UNPROTECT(1);
+    }
+
     STRI__UNPROTECT_ALL
     return ret;
     STRI__ERROR_HANDLER_END(if (str_text) utext_close(str_text);)
-    }
+}
 
 
 /**
@@ -246,13 +257,16 @@ SEXP stri_match_last_regex(SEXP str, SEXP pattern, SEXP cg_missing, SEXP opts_re
  *
  * @version 1.0-2 (Marek Gagolewski, 2016-01-29)
  *    Issue #214: allow a regex pattern like `.*`  to match an empty string
+ *
+ * @version 1.7.1 (Marek Gagolewski, 2021-06-19)
+ *    #153: named capture groups
  */
 SEXP stri_match_all_regex(SEXP str, SEXP pattern, SEXP omit_no_match, SEXP cg_missing, SEXP opts_regex)
 {
     bool omit_no_match1 = stri__prepare_arg_logical_1_notNA(omit_no_match, "omit_no_match");
-    PROTECT(str = stri_prepare_arg_string(str, "str")); // prepare string argument
-    PROTECT(pattern = stri_prepare_arg_string(pattern, "pattern")); // prepare string argument
-    PROTECT(cg_missing = stri_prepare_arg_string_1(cg_missing, "cg_missing"));
+    PROTECT(str = stri__prepare_arg_string(str, "str")); // prepare string argument
+    PROTECT(pattern = stri__prepare_arg_string(pattern, "pattern")); // prepare string argument
+    PROTECT(cg_missing = stri__prepare_arg_string_1(cg_missing, "cg_missing"));
     R_len_t vectorize_length = stri__recycling_rule(true, 2, LENGTH(str), LENGTH(pattern));
 
     StriRegexMatcherOptions pattern_opts =
@@ -268,6 +282,7 @@ SEXP stri_match_all_regex(SEXP str, SEXP pattern, SEXP omit_no_match, SEXP cg_mi
     SEXP ret;
     STRI__PROTECT(ret = Rf_allocVector(VECSXP, vectorize_length));
 
+    R_len_t last_i = -1;
     for (R_len_t i = pattern_cont.vectorize_init();
             i != pattern_cont.vectorize_end();
             i = pattern_cont.vectorize_next(i))
@@ -281,10 +296,17 @@ SEXP stri_match_all_regex(SEXP str, SEXP pattern, SEXP omit_no_match, SEXP cg_mi
 
         UErrorCode status = U_ZERO_ERROR;
         RegexMatcher *matcher = pattern_cont.getMatcher(i); // will be deleted automatically
-        int pattern_cur_groups = matcher->groupCount();
+        R_len_t pattern_cur_groups = matcher->groupCount();
+
+        SEXP cur_res, dimnames;  // all 2 will be PROTECT'd below
+        STRI__PROTECT(dimnames = pattern_cont.getCaptureGroupRDimnames(i, last_i, ret));
+        last_i = i;
 
         if ((str_cont).isNA(i)) {
-            SET_VECTOR_ELT(ret, i, stri__matrix_NA_STRING(1, pattern_cur_groups+1));
+            STRI__PROTECT(cur_res = stri__matrix_NA_STRING(1, pattern_cur_groups+1));
+            if (!isNull(dimnames)) Rf_setAttrib(cur_res, R_DimNamesSymbol, dimnames);
+            SET_VECTOR_ELT(ret, i, cur_res);
+            STRI__UNPROTECT(2);  // cur_res, dimnames
             continue;
         }
 
@@ -308,13 +330,17 @@ SEXP stri_match_all_regex(SEXP str, SEXP pattern, SEXP omit_no_match, SEXP cg_mi
 
         R_len_t noccurrences = (R_len_t)occurrences.size()/(pattern_cur_groups+1);
         if (noccurrences <= 0) {
-            SET_VECTOR_ELT(ret, i, stri__matrix_NA_STRING(omit_no_match1?0:1, pattern_cur_groups+1));
+            STRI__PROTECT(cur_res = stri__matrix_NA_STRING(omit_no_match1?0:1, pattern_cur_groups+1));
+            if (!isNull(dimnames)) Rf_setAttrib(cur_res, R_DimNamesSymbol, dimnames);
+            SET_VECTOR_ELT(ret, i, cur_res);
+            STRI__UNPROTECT(2);  // cur_res, dimnames
             continue;
         }
 
-        const char* str_cur_s = str_cont.get(i).c_str();
-        SEXP cur_res;
         STRI__PROTECT(cur_res = Rf_allocMatrix(STRSXP, noccurrences, pattern_cur_groups+1));
+        if (!isNull(dimnames)) Rf_setAttrib(cur_res, R_DimNamesSymbol, dimnames);
+
+        const char* str_cur_s = str_cont.get(i).c_str();
         deque< pair<R_len_t, R_len_t> >::iterator iter = occurrences.begin();
         for (R_len_t j = 0; iter != occurrences.end(); ++j) {
             pair<R_len_t, R_len_t> curo = *iter;
@@ -329,8 +355,9 @@ SEXP stri_match_all_regex(SEXP str, SEXP pattern, SEXP omit_no_match, SEXP cg_mi
                                    Rf_mkCharLenCE(str_cur_s+curo.first, curo.second-curo.first, CE_UTF8));
             }
         }
+
         SET_VECTOR_ELT(ret, i, cur_res);
-        STRI__UNPROTECT(1);
+        STRI__UNPROTECT(2);  // cur_res, dimnames
     }
 
     if (str_text) {
@@ -340,4 +367,4 @@ SEXP stri_match_all_regex(SEXP str, SEXP pattern, SEXP omit_no_match, SEXP cg_mi
     STRI__UNPROTECT_ALL
     return ret;
     STRI__ERROR_HANDLER_END(if (str_text) utext_close(str_text);)
-    }
+}
